/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

public class ConstructedDataAttribute extends ModelNode {

	/**
	 * 
	 * @param objectReference
	 * @param fc
	 * @param children
	 *            can be of type CONSTRUCTED_DATA_ATTRIBUTE and/or
	 *            BASIC_DATA_ATTRIBUTE
	 */
	public ConstructedDataAttribute(ObjectReference objectReference, FunctionalConstraint fc, List<ModelNode> children) {
		this.objectReference = objectReference;
		this.fc = fc;
		this.children = new LinkedHashMap<String, ModelNode>();
		for (ModelNode child : children) {
			this.children.put(child.getNodeName(), child);
		}
	}

	@Override
	public ConstructedDataAttribute copy() {
		List<ModelNode> subDataAttributesCopy = new ArrayList<ModelNode>();
		for (ModelNode subDA : children.values()) {
			subDataAttributesCopy.add(subDA.copy());
		}
		return new ConstructedDataAttribute(this.getReference(), this.fc, subDataAttributesCopy);
	}

}
