/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

public class DataSet extends ModelNode {
//	private final List<ModelNode> memberRef;

	private boolean deletable;

//	public DataSet() {
//	}

//	public DataSet(ObjectReference objectReference) {
//		this.objectReference = objectReference;
//	}
	
	public DataSet(ObjectReference objectReference, List<ModelNode> members, boolean deletable){
		this.children = new LinkedHashMap<String, ModelNode>();		
		this.objectReference = objectReference;
//		memberRef = members;		
		this.deletable = deletable;
		
		for(ModelNode node : members){
			this.children.put(node.getReference().getName(), node);
		}
	}

//	public List<ModelNode> getDataObjects() {
//		return memberRef;
//	}

	@Override
	public List<ModelNode> getChildren() {
		if (children == null) {
			return null;
		}
		return new ArrayList<ModelNode>(children.values());
	}

	public void setFunctionalConstraint(FunctionalConstraint fc) {
		throw new UnsupportedOperationException("Cannot set FC on DataSet");
	}

	@Override
	public DataSet copy() {
		List<ModelNode> membersCopy = new ArrayList<ModelNode>();
		for (ModelNode node : children.values()) {			
			membersCopy.add(node.copy());
		}
		return new DataSet(this.objectReference, membersCopy, deletable);
	}

	@Override
	public ModelNode getChild(String childName, FunctionalConstraint fc) {
		for (ModelNode child : this) {
			if (child.getNodeName().equals(childName)) {
				return child;
			}
		}
		return null;
	}

//	@Override
//	protected Iterator<Iterator<? extends ModelNode>> getIterators() {
//		List<Iterator<? extends ModelNode>> iterators = new ArrayList<Iterator<? extends ModelNode>>();
//		if (memberRef != null) {
//			iterators.add(memberRef.iterator());
//		}
//		return iterators.iterator();
//	}

	// Note: it is deletable if it is persistent and non persistent. Configured
	// data sets are not deletable
	// Before the data set is deleted, it must have isDeletable = true AND there
	// are no reports/logs using it
	public boolean isDeletable() {
		return deletable;
	}

//	public void setDeletable(boolean deletable) {
//		this.deletable = deletable;
//	}

//	@Override
//	public List<BasicDataAttribute> getBasicDataAttributes() {
//		// TODO Auto-generated method stub
//		
//		return null;
//	}

}
