/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.ArrayList;
import java.util.EnumMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.openiec61850.common.model.report.ReportControlBlock;

/**
 * 
 * @author Michael Zillgith
 * @author Stefan Feuerhahn
 * @author Tobias Weidelt
 * 
 */
public class LogicalNode extends ModelNode {

	private final Map<String, DataSet> dataSets = new LinkedHashMap<String, DataSet>();
	private final List<ReportControlBlock> reportControlBlocks = new ArrayList<ReportControlBlock>();
	// private final List<LogControlBlock> logControlBlocks = new
	// ArrayList<LogControlBlock>();
	private final Map<FunctionalConstraint, Map<String, DataObject>> fcDataObjects = new EnumMap<FunctionalConstraint, Map<String, DataObject>>(
			FunctionalConstraint.class);

	public LogicalNode(ObjectReference objectReference, List<DataObject> dataObjects, List<DataObject> fcDataObjects,
			List<DataSet> dataSets) {
		children = new LinkedHashMap<String, ModelNode>();
		this.objectReference = objectReference;
		if (dataObjects != null) {
			for (DataObject dataObject : dataObjects) {
				this.children.put(dataObject.getReference().getName(), dataObject);
			}
		}

		if (fcDataObjects == null) {
			for (DataObject dataObject : dataObjects) {
				for (DataObject fcDataObject : dataObject.getFCDataObjects()) {
					if (this.fcDataObjects.get(fcDataObject.getFunctionalConstraint()) == null) {
						this.fcDataObjects.put(fcDataObject.getFunctionalConstraint(),
								new LinkedHashMap<String, DataObject>());
					}
					this.fcDataObjects.get(fcDataObject.getFunctionalConstraint()).put(
							fcDataObject.getReference().getName(), fcDataObject);
				}
			}
		}
		else {
			for (DataObject fcDataObject : fcDataObjects) {
				if (this.fcDataObjects.get(fcDataObject.getFunctionalConstraint()) == null) {
					this.fcDataObjects.put(fcDataObject.getFunctionalConstraint(),
							new LinkedHashMap<String, DataObject>());
				}
				this.fcDataObjects.get(fcDataObject.getFunctionalConstraint()).put(
						fcDataObject.getReference().getName(), fcDataObject);
			}
		}

		if (dataSets != null) {
			for (DataSet ds : dataSets) {
				this.dataSets.put(ds.getReference().getName(), ds);
			}
		}
	}

	@Override
	public LogicalNode copy() {

		List<DataObject> dataObjectsCopy = new ArrayList<DataObject>();
		for (ModelNode obj : children.values()) {
			dataObjectsCopy.add((DataObject) obj.copy());
		}

		List<DataSet> dataSetsCopy = new ArrayList<DataSet>();
		for (DataSet dataSet : dataSets.values()) {
			dataSetsCopy.add(dataSet.copy());
		}

		// for (ReportControlBlock rcb : reportControlBlocks) {
		// addReportControlBlock(rcb.copyDeep());
		// }
		// for (LogControlBlock lcb : logControlBlocks) {
		// addLogControlBlock(lcb.copyDeep());
		// }

		LogicalNode copy = new LogicalNode(objectReference, dataObjectsCopy, null, dataSetsCopy);
		return copy;
	}

	public List<DataSet> getDataSets() {
		return new ArrayList<DataSet>(dataSets.values());
	}

	public void deleteDataSet(String dsName) {
		dataSets.remove(dsName);

	}

	// public void setDataSets(List<DataSet> dataSets) {
	// this.dataSets = dataSets;
	// }

	public void addDataSet(DataSet ds) {
		dataSets.put(ds.getReference().getName(), ds);
	}

	public void addReportControlBlock(ReportControlBlock rcb) {
		reportControlBlocks.add(rcb);
	}

	public DataSet getDataSet(String name) {
		return dataSets.get(name);
	}

	public List<ReportControlBlock> getReportControlBlocks() {
		return reportControlBlocks;
	}
	
	// public void addLogControlBlock(LogControlBlock lcb) {
	// logControlBlocks.add(lcb);
	// }
	//
	// public List<LogControlBlock> getLogControlBlocks() {
	// return logControlBlocks;
	// }

	public List<DataObject> getChildren(FunctionalConstraint fc) {
		if (fcDataObjects.get(fc) == null) {
			return null;
		}
		return new ArrayList<DataObject>(fcDataObjects.get(fc).values());
	}

	public Map<FunctionalConstraint, Map<String, DataObject>> getFCChildren() {
		return fcDataObjects;
	}

	public ModelNode getChild(String childName, FunctionalConstraint fc) {
		return fcDataObjects.get(fc).get(childName);
	}

}
