/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * 
 * @author Michael Zillgith
 * @author Stefan Feuerhahn
 * @author Tobias Weidelt
 * 
 */
final public class Server extends ModelNode {

	private String authentication = null;

	public Server(List<LogicalDevice> logicalDevices) {
		children = new LinkedHashMap<String, ModelNode>();
		objectReference = null;
		for (LogicalDevice logicalDevice : logicalDevices) {
			this.children.put(logicalDevice.getReference().getName(), logicalDevice);
		}
	}

	// TODO authentication necessary?
	public String getAuthentication() {
		return authentication;
	}

	public void setAuthentication(String authentication) {
		this.authentication = authentication;
	}

	@Override
	public Server copy() {
		List<LogicalDevice> childCopies = new ArrayList<LogicalDevice>(children.size());
		for (ModelNode childNode : children.values()) {
			childCopies.add((LogicalDevice) childNode.copy());
		}
		return new Server(childCopies);

	}

	@Override
	public String toString() {
		return "Server";
	}

}
