/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.openiec61850.common.model.datypes.commonacsi;

import org.openiec61850.common.model.BasicDataAttribute;
import org.openiec61850.common.model.FunctionalConstraint;
import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.common.model.datypes.DAType;

/**
 * 
 * Quality basic type according to 61850-7-3 table 1
 * 
 * @author mzillgit
 * 
 */
public class Quality extends BasicDataAttribute {

	public Quality(ObjectReference objectReference, FunctionalConstraint fc, String sAddr, boolean qchg) {
		this.objectReference = objectReference;
		this.fc = fc;
		this.basicType = DAType.QUALITY;
		this.sAddr = sAddr;
		this.qchg = qchg;
	}

	public enum Validity {
		GOOD(0), INVALID(1), RESERVED(2), QUESTIONABLE(3);
		private final int value;

		private Validity(int value) {
			this.value = value;
		}

		public int getIntValue() {
			return value;
		}
	}

	public enum Source {
		PROCESS(0), SUBSTITUTED(1);
		private final int value;

		private Source(int value) {
			this.value = value;
		}

		public int getIntValue() {
			return value;
		}
	}

	public Validity validity = Validity.GOOD;

	/* detailQual attribute */
	public boolean overflow = false;
	public boolean outOfRange = false;
	public boolean badReference = false;
	public boolean oscillatory = false;
	public boolean failure = false;
	public boolean oldData = false;
	public boolean inconsistent = false;
	public boolean inaccurate = false;

	public Source source = Source.PROCESS;

	/* test attribute */
	public boolean test = false;

	/* operatorBlocked attribute */
	public boolean operatorBlocked = false;

	// return the validity value??
	public Integer getValue() {
		return new Integer(this.validity.getIntValue());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (badReference ? 1231 : 1237);
		result = prime * result + (failure ? 1231 : 1237);
		result = prime * result + (inaccurate ? 1231 : 1237);
		result = prime * result + (inconsistent ? 1231 : 1237);
		result = prime * result + (oldData ? 1231 : 1237);
		result = prime * result + (operatorBlocked ? 1231 : 1237);
		result = prime * result + (oscillatory ? 1231 : 1237);
		result = prime * result + (outOfRange ? 1231 : 1237);
		result = prime * result + (overflow ? 1231 : 1237);
		result = prime * result + ((source == null) ? 0 : source.hashCode());
		result = prime * result + (test ? 1231 : 1237);
		result = prime * result + ((validity == null) ? 0 : validity.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		Quality other = (Quality) obj;
		if (badReference != other.badReference) {
			return false;
		}
		if (failure != other.failure) {
			return false;
		}
		if (inaccurate != other.inaccurate) {
			return false;
		}
		if (inconsistent != other.inconsistent) {
			return false;
		}
		if (oldData != other.oldData) {
			return false;
		}
		if (operatorBlocked != other.operatorBlocked) {
			return false;
		}
		if (oscillatory != other.oscillatory) {
			return false;
		}
		if (outOfRange != other.outOfRange) {
			return false;
		}
		if (overflow != other.overflow) {
			return false;
		}
		if (source != other.source) {
			return false;
		}
		if (test != other.test) {
			return false;
		}
		if (validity != other.validity) {
			return false;
		}
		return true;
	}

	// @Override
	// public String toString() {
	// return validity + " " + source;
	// }

	@Override
	public void setDefault() {
		validity = Validity.GOOD;

		/* detailQual attribute */
		overflow = false;
		outOfRange = false;
		badReference = false;
		oscillatory = false;
		failure = false;
		oldData = false;
		inconsistent = false;
		inaccurate = false;

		source = Source.PROCESS;

		/* test attribute */
		test = false;

		/* operatorBlocked attribute */
		operatorBlocked = false;
	}

	@Override
	public Quality copy() {
		// TODO Auto-generated method stub
		return new Quality(this.objectReference, this.fc, this.sAddr, this.qchg);

		// return null;
	}
}
