/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model.datypes.commonacsi;

import java.util.Calendar;
import java.util.Date;

/**
 * See 61850-7-2 chapter TypeDefinitions/BasicTypes
 * 
 * @author bertram
 */
public class TimeQuality {
	public static TimeQuality DEFAULT = new TimeQuality();
	public static TimeQuality INVALID = new TimeQuality(true);
	/**
	 * The value TRUE of the attribute LeapSecondsKnown shall indicate that the
	 * value for SecondSinceEpoch takes into account all leap seconds occurred.
	 * If it is FALSE then the value does not take into account the leap seconds
	 * that occurred before the initialization of the time source of the device.
	 * 
	 * Java {@link Date} and {@link Calendar} objects do handle leap seconds, so
	 * this is usually true.
	 */
	public boolean leapSecondsknown = true;
	/**
	 * The attribute clockFailure shall indicate that the time source of the
	 * sending device is unreliable. The value of the TimeStamp shall be
	 * ignored.
	 */
	public boolean clockFailure = false;
	/**
	 * The attribute clockNotSynchronized shall indicate that the time source of
	 * the sending device is not synchronized with the external UTC time.
	 */
	public boolean clockNotSynchronized = false;
	/**
	 * The attribute TimeAccuracy shall represent the time accuracy class of the
	 * time source of the sending device relative to the external UTC time. The
	 * timeAccuracy classes shall represent the number of significant bits in
	 * the FractionOfSecond
	 * 
	 * If the time is set via Java {@link Date} objects, the accuracy is 1 ms,
	 * that is a timeAccuracy value of 10.
	 */
	public int timeAccuracy = 10;

	public TimeQuality() {

	}

	public TimeQuality(boolean clockFailure) {
		this.clockFailure = clockFailure;
	}
}
