package org.openiec61850.acsi;

import java.util.HashSet;
import java.util.Set;

import org.junit.Assert;
import org.junit.Test;
import org.junit.internal.matchers.IsCollectionContaining;
import org.openiec61850.common.model.ObjectReference;

/**
 * @author tweidelt
 * 
 */
public class ObjectReferenceTest {

	@Test
	public void testToString() {
		String value = "abc.def/ghi.jkl";
		ObjectReference ref = new ObjectReference(value);
		Assert.assertEquals(value, ref.toString());
	}

	@Test
	public void testGetNameMethods() {
		ObjectReference ref = new ObjectReference("abc.def/ghi.jkl");
		Assert.assertEquals("abc.def", ref.get(0));
		Assert.assertEquals("ghi", ref.get(1));
		Assert.assertEquals("jkl", ref.get(2));

		ref = new ObjectReference("abc.def");
		Assert.assertEquals("abc.def", ref.getName()); // special case with no
		// LogicalNode

		ref = new ObjectReference("abc.def/ghi");
		Assert.assertEquals("ghi", ref.getName());

		ref = new ObjectReference("abc.def/ghi.jkl");
		Assert.assertEquals("jkl", ref.getName());

		ref = new ObjectReference("abc.def/ghi.jkl.mno");
		Assert.assertEquals("mno", ref.getName());
	}

	@Test
	public void testParser() {
		ObjectReference ref = new ObjectReference("abc.def/ghi.jkl");
		Set<String> names = new HashSet<String>();
		for (String name : ref) {
			System.out.println(name);
			names.add(name);
		}
		Assert.assertThat(names, IsCollectionContaining.hasItem("abc.def"));
		Assert.assertThat(names, IsCollectionContaining.hasItem("ghi"));
		Assert.assertThat(names, IsCollectionContaining.hasItem("jkl"));
	}

	@Test
	public void testGetParentRef() {
		ObjectReference ref = new ObjectReference("abc/def.ghi");
		Assert.assertEquals("abc/def", ref.getParentRef());
		ref = new ObjectReference("abc/def");
		Assert.assertEquals("abc", ref.getParentRef());
	}

}
