/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.server;

import java.util.Collection;
import java.util.List;

import org.openiec61850.common.ACSIClass;
import org.openiec61850.common.ServiceError;
import org.openiec61850.common.model.FunctionalConstraint;
import org.openiec61850.common.model.ModelNode;
import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.common.model.report.UnbufferedReportContrlBlock;
import org.openiec61850.server.scsm.SCSMConnectionHandler;

public interface ServerACSI {
	public enum ServerDirectoryType {
		LOGICAL_DEVICES, FILES
	}

	// TODO add get DataDirectory etc...

	Association associate(SCSMConnectionHandler clientHandler, String auth) throws ServiceError;

	void abort(Association association, String reason) throws ServiceError;

	void release(Association association) throws ServiceError;

	/**
	 * 
	 * @return list of names/references of logical devices or file names
	 */
	List<String> getServerDirectory(Association association, ServerDirectoryType serverDirectoryType)
			throws ServiceError;

	List<String> getLogicalDeviceDirectory(Association association, String logicalDeviceName) throws ServiceError;

	List<String> getLogicalNodeDirectory(Association assoc, String logicalNodeReference, ACSIClass acsiClass)
			throws ServiceError;

	ModelNode getDataDefinition(Association association, ObjectReference objectReference, FunctionalConstraint fc)
			throws ServiceError;

	/**
	 * 
	 * @param association
	 * @param objectReference
	 * @param fc
	 *            Filter for Functional Constraint
	 * @return Functionally constraint ModelNode which contains the requested
	 *         values in its leave nodes
	 * @throws ServiceError
	 */
	ModelNode getDataValues(Association association, String objectReference, FunctionalConstraint fc)
			throws ServiceError;

	void setDataValues(Association association, ModelNode modelNode) throws ServiceError;

	/**
	 * 
	 * @param association
	 * @param ref
	 *            ObjectReference of the SGCB
	 * @param sGN
	 *            the number of the setting group to edit
	 * @throws ServiceError
	 */
	void selectEditSG(Association association, String ref, int sGN) throws ServiceError;

	// Data set functions
	void createDataSet(Association association, String dataSetName, List<String> dsMemberList,
			List<FunctionalConstraint> fc) throws ServiceError;

	ModelNode getDataSetDirectory(Association association, String dataSetname) throws ServiceError;

	int[] deleteDataSet(Association association, String dataSetName) throws ServiceError;

	List<ModelNode> getDataSetValues(Association association, Collection<ModelNode> dataSetMembers) throws ServiceError;

	List<ServiceError> setDataSetValues(Association association, List<ModelNode> members);

	void setURCBValues(Association association, UnbufferedReportContrlBlock urcb, ModelNode value) throws ServiceError;

	ModelNode getRCB(Association association, ObjectReference objRef) throws ServiceError;

}
