/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.server;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import org.openiec61850.server.data.ConfigurationException;
import org.openiec61850.server.data.PropertiesParser;
import org.openiec61850.server.scl.ICDParser;
import org.openiec61850.server.scl.SCLParserException;
import org.openiec61850.server.scsm.SCSMServerSAP;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class StandaloneServer implements SAPStopListener {

	// static int mmsPort = 0;
	// static int soapPort = 0;
	// static String ip = null;
	// static String icdFileName = null;
	// static String configFileName = null;
	// static Level logLevel = Level.INFO;

	// MMSServerSAP mmsServerSAP = null;
	// SoapServerConnectorJetty soapServerSAP = null;

	private List<SCSMServerSAP> scsmServerSAPs = new LinkedList<SCSMServerSAP>();

	private static Logger logger = LoggerFactory.getLogger(StandaloneServer.class);

	public static void main(String[] args) throws IOException, ConfigurationException {
		if (args.length != 1) {
			System.out.println("usage: org.openiec61850.server.Server <properties file>");
			return;
		}

		new StandaloneServer(args);
	}

	StandaloneServer(String[] args) throws IOException, ConfigurationException {

		// Runtime.getRuntime().addShutdownHook(new Thread() {
		// public void run() {
		// if (scsmServerSAPs != null) {
		// for (SCSMServerSAP scsmServerSAP : scsmServerSAPs) {
		// scsmServerSAP.stopListening();
		// }
		// }
		// logger.error("Server was killed.");
		//
		// }
		// });

		// try {
		// parseCommandLine(args);
		// } catch (Exception e) {
		// return;
		// }
		// if (mmsPort == 0 && soapPort == 0) {
		// System.err.println("Neither MMS nor SOAP have been enabled.");
		// return;
		// }

		// setuplogger();

		PropertiesParser propertiesParser = new PropertiesParser();
		try {
			propertiesParser.parse(args[0]);
		} catch (Exception e) {
			System.err.println("Error parsing properties file: " + e.getMessage());
			return;
		}

		logger.debug(System.getProperty("java.vm.name"));

		ICDParser icdParser = new ICDParser();
		try {
			icdParser.parse(propertiesParser.getICDFilePath());
		} catch (SCLParserException e) {
			System.err.println("Error parsing ICD file: " + e.getMessage());
			return;
		}

		IED ied = icdParser.getIED();

		List<AccessPoint> initializedAccessPoints = new LinkedList<AccessPoint>();
		for (AccessPoint accessPoint : ied.getAccessPoints()) {
			String dataSourceClassName = propertiesParser.getDataSourceClassName(accessPoint.getName());
			if (dataSourceClassName == null) {
				System.err.println("No DataSource has been configured for the AccessPoint \"" + accessPoint.getName()
						+ "\" in the properties file.");
			}
			else {
				try {
					accessPoint.initDataSource(propertiesParser.getDataSourceClassName(accessPoint.getName()));
					initializedAccessPoints.add(accessPoint);
				} catch (ConfigurationException e) {
					System.out.println("Error initializing DataSource: " + e.getMessage());
					e.printStackTrace();
					return;
				}
			}
		}

		if (initializedAccessPoints.size() == 0) {
			System.err.println("No AccessPoint could be connected to a DataSource. Will exit.");
			return;
		}

		for (String sapPropName : propertiesParser.sapNames) {
			logger.info("configuring SAP: " + sapPropName);
			SCSMServerSAP scsmServerSAP = null;
			try {
				scsmServerSAP = (SCSMServerSAP) Class.forName(propertiesParser.getSAPClassName(sapPropName))
						.newInstance();
			} catch (Exception e) {
				throw new ConfigurationException("Exception instantiating ServerSAP: "
						+ propertiesParser.getSAPClassName(sapPropName) + e);
			}

			boolean foundAccessPointForSAP = false;
			for (AccessPoint accessPoint : initializedAccessPoints) {
				if (accessPoint.getName().equals(propertiesParser.getAPName(sapPropName))) {
					scsmServerSAP.init(accessPoint, this, propertiesParser.properties, sapPropName);
					foundAccessPointForSAP = true;
				}
			}

			if (foundAccessPointForSAP == false) {
				logger.error("not AccessPoint found");
				return;
			}

			scsmServerSAPs.add(scsmServerSAP);

		}

		for (SCSMServerSAP scsmServerSAP : scsmServerSAPs) {
			scsmServerSAP.startListening();
		}

		// if (mmsPort > 0) {
		// mmsServerSAP = new MMSServerSAP(mmsPort, 0,
		// InetAddress.getByName(ip), ied.getAccessPoints().get(0), this);
		// mmsServerSAP.setMaxConnections(10);
		// logger.info("created MMSServerSAP on port {}", mmsPort);
		//
		// mmsServerSAP.startListening();
		// }
		// if (soapPort > 0) {
		// soapServerSAP = new SoapServerConnectorJetty(ip, soapPort,
		// ied.getAccessPoints().get(0));
		// soapServerSAP.startListening();
		// }

		logger.info("IED Server is running");
	}

	// private static void setuplogger() {
	// ch.qos.logback.classic.Logger rootLogger =
	// (ch.qos.logback.classic.Logger) LoggerFactory
	// .getLogger(Logger.ROOT_LOGGER_NAME);
	// rootLogger.setLevel(logLevel);
	// }

	// @SuppressWarnings("static-access")
	// private static void parseCommandLine(String[] args) throws ParseException
	// {
	//
	// Options options = new Options();
	// options.addOption(OptionBuilder.withDescription("print this help text").withLongOpt("help").create("h"));
	// options.addOption(OptionBuilder
	// .withDescription("logger level (debug, info, warn, error, or off), default: info").hasArg()
	// .withLongOpt("log").withArgName("Log level").create('l'));
	// options.addOption(OptionBuilder.withDescription("path of the ICD file to use").hasArg().isRequired()
	// .withLongOpt("icd").withArgName("SCL file").create('s'));
	// options.addOption(OptionBuilder.withDescription("path to the config properties file").hasArg().isRequired()
	// .withLongOpt("config").withArgName("config file").create('c'));
	// options.addOption(OptionBuilder.withDescription("local ip to bind to - default: bind to any ip")
	// .withLongOpt("ip").hasArg().create("i"));
	// options.addOption(OptionBuilder.withDescription("enable MMS protocol and set port number, e.g. 102")
	// .withLongOpt("mms").hasArg().withArgName("port number").withType(Number.class).create('m'));
	// options.addOption(OptionBuilder.withDescription("enable SOAP protocol and set port number").withLongOpt("soap")
	// .hasArg().withArgName("port number").withType(Number.class).create('w'));
	//
	// CommandLineParser cmdLineParser = new GnuParser();
	// CommandLine cmd;
	// try {
	// cmd = cmdLineParser.parse(options, args);
	//
	// if (cmd.hasOption("h")) {
	// throw new ParseException("print help");
	// }
	// if (cmd.hasOption('l')) {
	//
	// String logLevelOptionVal = cmd.getOptionValue('l');
	// if (logLevelOptionVal.equals("debug")) {
	// logLevel = Level.DEBUG;
	// }
	// else if (logLevelOptionVal.equals("info")) {
	// logLevel = Level.INFO;
	// }
	// else if (logLevelOptionVal.equals("warn")) {
	// logLevel = Level.WARN;
	// }
	// else if (logLevelOptionVal.equals("error")) {
	// logLevel = Level.ERROR;
	// }
	// else if (logLevelOptionVal.equals("off")) {
	// logLevel = Level.OFF;
	// }
	// else {
	// System.out.println("Unknown log level");
	// throw new ParseException("");
	// }
	// }
	// if (cmd.hasOption('m')) {
	// mmsPort = ((Number) cmd.getParsedOptionValue("m")).intValue();
	// }
	// if (cmd.hasOption("w")) {
	// soapPort = ((Number) cmd.getParsedOptionValue("w")).intValue();
	// }
	// if (cmd.hasOption("i")) {
	// ip = cmd.getOptionValue("i");
	// }
	// icdFileName = cmd.getOptionValue("s");
	// configFileName = cmd.getOptionValue("c");
	//
	// } catch (ParseException e) {
	// HelpFormatter formatter = new HelpFormatter();
	// formatter.printHelp(IED.class.getName(), options, true);
	// throw e;
	// }
	// }

	public void serverStoppedListening(SCSMServerSAP scsmServerSAP) {
		logger.error("An SAP stopped listening");

	}

}
