/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.server.log;

import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.common.model.datypes.commonacsi.Timestamp;
import org.openiec61850.common.model.log.LogEntry;
import org.openiec61850.common.model.report.ClientReportState;
import org.openiec61850.common.model.report.Report;
import org.openiec61850.server.data.ConfigurationException;

/**
 * Interface for classes that can handle persistent storage of logging and
 * buffered reporting
 */
public interface DataStorage {

	public void init() throws ConfigurationException;

	/**
	 * writes {@link LogEntry} to persistent storage and sets
	 * {@link LogEntry#entryId}
	 * 
	 * @param log
	 */
	public void writeLogEntry(LogEntry log);

	public Collection<LogEntry> queryLogByTime(ObjectReference logReference, Timestamp rangeStartTime,
			Timestamp rangeStopTime);

	public Collection<LogEntry> queryLogAfter(ObjectReference logReference, Timestamp rangeStartTime, long startEntryId);

	public void purgeLog(Date upToDate);

	/**
	 * Writes {@link Report} to persistent storage and sets
	 * {@link Report#entryId}
	 * 
	 * @param report
	 */
	public void writeReport(Report report);

	public Collection<Report> readReport(String reportId, long fromEntryId);

	public void purgeReport();

	public List<ClientReportState> getClientReportStates();

	public void writeClientReportStates(List<ClientReportState> persistentStates);

}
