/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.server.log;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.common.model.datypes.commonacsi.Timestamp;
import org.openiec61850.common.model.log.LogEntry;
import org.openiec61850.common.model.report.ClientReportState;
import org.openiec61850.common.model.report.Report;
import org.openiec61850.server.data.ConfigurationException;

/**
 * Simple {@link DataStorage} implementation that stores data in memory
 * 
 * @author bertram
 */
public class InMemoryDataStorage implements DataStorage {
	private final List<LogEntry> logEntries = new ArrayList<LogEntry>();
	private List<ClientReportState> clientReportStates = new ArrayList<ClientReportState>();
	private final List<Report> reports = new ArrayList<Report>();
	private long reportEntryID = 1L;
	private long logEntryID = 1L;

	public void init() throws ConfigurationException {
	}

	public void writeLogEntry(LogEntry log) {
		logEntries.add(log);
		log.setEntryId(logEntryID++);
		if (logEntries.size() > 100) {
			logEntries.remove(0);
		}
	}

	public Collection<LogEntry> queryLogByTime(ObjectReference logReference, Timestamp rangeStartTime,
			Timestamp rangeStopTime) {
		List<LogEntry> result = new ArrayList<LogEntry>();
		for (LogEntry le : logEntries) {
			if (le.getLogRef().toString().equals(logReference.toString())
					&& le.getEntryTime().compareTo(rangeStartTime) > 0
					&& le.getEntryTime().compareTo(rangeStopTime) < 0) {
				result.add(le);
			}
		}
		return result;
	}

	public Collection<LogEntry> queryLogAfter(ObjectReference logReference, Timestamp rangeStartTime, long startEntryId) {
		List<LogEntry> result = new ArrayList<LogEntry>();
		for (LogEntry le : logEntries) {
			if (le.getLogRef().toString().equals(logReference.toString())
					&& le.getEntryTime().compareTo(rangeStartTime) > 0 && le.getEntryId() > startEntryId) {
				result.add(le);
			}
		}
		return result;
	}

	public void purgeLog(Date date) {
		logEntries.clear();
	}

	public void writeReport(Report report) {
		report.setEntryId(reportEntryID++);
		reports.add(report);
		if (reports.size() > 100) {
			reports.remove(0);
		}
	}

	public void purgeReport() {
		reports.clear();
	}

	public Collection<Report> readReport(String reportId, long fromEntryId) {
		List<Report> result = new ArrayList<Report>();
		for (Report report : reports) {
			if (report.getEntryId() > fromEntryId && report.getRptId().equals(reportId)) {
				result.add(report);
			}
		}
		return result;
	}

	public List<ClientReportState> getClientReportStates() {
		return clientReportStates;
	}

	public void writeClientReportStates(List<ClientReportState> persistentStates) {
		this.clientReportStates = persistentStates;
	}
}