/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.server.scl;

import org.openiec61850.common.model.datypes.DAType;

public class Util {
	static boolean parseBooleanValue(String value) throws SCLParserException {
		if (value.equals("true")) {
			return true;
		}
		else if (value.equals("false")) {
			return false;
		}
		else {
			throw new SCLParserException("Not a boolean value");
		}
	}

	static boolean isBasicType(String bType) throws SCLParserException {
		if (bType.equals("BOOLEAN")) {
			return true;
		}
		if (bType.equals("INT8")) {
			return true;
		}
		if (bType.equals("INT16")) {
			return true;
		}
		if (bType.equals("INT24")) {
			return true;
		}
		if (bType.equals("INT32")) {
			return true;
		}
		if (bType.equals("INT8U")) {
			return true;
		}
		if (bType.equals("INT16U")) {
			return true;
		}
		if (bType.equals("INT24U")) {
			return true;
		}
		if (bType.equals("INT32U")) {
			return true;
		}
		if (bType.equals("FLOAT32")) {
			return true;
		}
		if (bType.equals("FLOAT64")) {
			return true;
		}
		if (bType.equals("Quality")) {
			return true;
		}
		if (bType.equals("Timestamp")) {
			return true;
		}
		if (bType.equals("VisString32")) {
			return true;
		}
		if (bType.equals("VisString64")) {
			return true;
		}
		if (bType.equals("VisString129")) {
			return true;
		}
		if (bType.equals("VisString255")) {
			return true;
		}
		if (bType.equals("Unicode255")) {
			return true;
		}
		if (bType.equals("Octet64")) {
			return true;
		}

		if (bType.equals("Struct")) {
			return false;
		}
		if (bType.equals("Enum")) {
			return true;
		}
		if (bType.equals("Dbpos")) {
			return true; /* changed by stefan */
		}
		
		if(bType.equals("Tcmd")){
			return true;
		}

		throw new SCLParserException("Invalid bType: " + bType);
	}

	public static DAType bTypeFromSCLString(String bType) throws SCLParserException {
		if (bType.equals("BOOLEAN")) {
			return DAType.BOOLEAN;
		}
		if (bType.equals("INT8")) {
			return DAType.INT8;
		}
		if (bType.equals("INT16")) {
			return DAType.INT16;
		}
//		if (bType.equals("INT24")) {
//			return BasicType.INT24;
//		}
		if (bType.equals("INT32")) {
			return DAType.INT32;
		}
		if (bType.equals("INT8U")) {
			return DAType.INT8U;
		}
		if (bType.equals("INT16U")) {
			return DAType.INT16U;
		}
//		if (bType.equals("INT24U")) {
//			return BasicType.INT24U;
//		}
		if (bType.equals("INT32U")) {
			return DAType.INT32U;
		}
		if (bType.equals("FLOAT32")) {
			return DAType.FLOAT32;
		}
//		if (bType.equals("FLOAT64")) {
//			return BasicType.FLOAT64;
//		}
		if (bType.equals("VisString32")) {
			return DAType.VISIBLE_STRING_32;
		}
		if (bType.equals("VisString64")) {
			return DAType.VISIBLE_STRING_64;
		}
		if (bType.equals("VisString129")) {
			return DAType.VISIBLE_STRING_129;
		}
		if (bType.equals("VisString255")) {
			return DAType.VISIBLE_STRING_255;
		}
		if (bType.equals("Unicode255")) {
			return DAType.UNICODE_STRING_255;
		}
		if (bType.equals("Octet64")) {
			return DAType.OCTET_STRING_64;
		}
		if (bType.equals("Quality")) {
			return DAType.QUALITY;
		}
		if (bType.equals("Timestamp")) {
			return DAType.TIMESTAMP;
		}
		if (bType.equals("Enum")) {
			return DAType.ENUMERATED;
		}
		if (bType.equals("Dbpos")) {
			return DAType.CODEDENUM;
		}
		
		if(bType.equals("Tcmd")){
			return DAType.CODEDENUM;
		}

		throw new SCLParserException("Invalid bType: " + bType);
	}
}
