/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.scsm.mms;

import java.io.UnsupportedEncodingException;

import org.openiec61850.common.model.FunctionalConstraint;
import org.openiec61850.common.model.ObjectReference;
import org.openiec61850.jmms.mms.asn1.ObjectName;
import org.openmuc.jasn1.ber.types.string.BerVisibleString;

public class MMSReference {

	// public static String toMMSItemId(String objRef, FunctionalConstraint fc)
	// {
	// return new ObjectReference(objRef).getMMSItemId(fc);
	// }
	//
	// public static String toMMSDomainId(String objRef) {
	// return new ObjectReference(objRef).getMMSDomainId();
	// }

	// /**
	// * @param fc
	// * @return Example:<br>
	// * value = E1Rly1/T1MMXU0.TotW.mag.f, fc = MX<br>
	// * Result = T1MMXU0$MX$TotW$mag$f
	// */
	// public String getMMSItemId(FunctionalConstraint fc) {
	// int firstSlash = objectReference.indexOf('/');
	//
	// if (firstSlash < 0) {
	// return "";
	// }
	//
	// int dotAfterSlash = objectReference.indexOf('.', firstSlash + 1);
	//
	// String lnName = getLNName();
	//
	// if (dotAfterSlash >= 0) {
	// String fcStr = "XX"; // TODO check if fc = XX is valid for MMS SCSM
	// if (fc != null) {
	// fcStr = fc.toString();
	// }
	// return lnName + "$" + fcStr + "$" +
	// objectReference.substring(dotAfterSlash + 1).replace('.', '$');
	// }
	// if (fc != null) {
	// return lnName + "$" + fc.toString();
	// }
	// return lnName;
	// }
	//
	// public String getMMSDomainId() {
	// return getLDName();
	// }

	/**
	 * @param mmsDomainId
	 * @param mmsItemId
	 * @return isEmpty Example:<br>
	 *         Input: domainId = E1Rly1, itemId = T1MMXU0$MX$TotW$mag$f<br>
	 *         Result = E1Rly1/T1MMXU0.TotW.mag.f *
	 */
	public static ObjectReference toObjectReference(String mmsDomainId,
			String mmsItemId) {
		if ((mmsItemId == null) || (mmsItemId.length() == 0)) {
			return new ObjectReference(mmsDomainId);
		}
		int firstDollar = mmsItemId.indexOf('$');
		if (firstDollar >= 0) {
			String lnName = mmsItemId.substring(0, firstDollar);

			int secondDollar = mmsItemId.indexOf('$', firstDollar + 1);
			if (secondDollar >= 0) {
				return new ObjectReference(mmsDomainId
						+ '/'
						+ lnName
						+ '.'
						+ mmsItemId.substring(secondDollar + 1).replace('$',
								'.'));
			}
			return new ObjectReference(mmsDomainId + '/'
					+ mmsItemId.substring(0, firstDollar));
		}
		return new ObjectReference(mmsDomainId + '/' + mmsItemId);
	}

	// public static String toObjectReference(ObjectName mmsObjectName) {
	// if (mmsObjectName.isDomain_specificSelected()) {
	// String mmsDomainId =
	// mmsObjectName.getDomain_specific().getDomainId().getValue();
	// String mmsItemId =
	// mmsObjectName.getDomain_specific().getItemId().getValue();
	// return toObjectReference(mmsDomainId, mmsItemId);
	// }
	// // TODO check for other ObjectName types?
	// return "";
	// }

	// for data sets since they do not have the FC in the mms name
	public static String toObjectReferenceDS(
			ObjectName mmsObjectName) {
		// if (mmsObjectName.isDomain_specificSelected()) {
		// String mmsDomainId =
		// mmsObjectName.getDomain_specific().getDomainId().getValue();
		// String mmsItemId =
		// mmsObjectName.getDomain_specific().getItemId().getValue();
		// return mmsDomainId + "/" + mmsItemId.replace('$', '.');
		if (mmsObjectName.domain_specific != null) {
			String mmsDomainId = mmsObjectName.domain_specific.domainId
					.toString();
			String mmsItemId = mmsObjectName.domain_specific.itemId.toString();
			return mmsDomainId + "/" + mmsItemId.replace('$', '.');
		}

		else if (mmsObjectName.aa_specific != null) {
			return mmsObjectName.aa_specific.toString(); // format is
			// "@DataSetName"
		}

		return "";
	}

	// public static ObjectName toMmsObjectName(String reference,
	// FunctionalConstraint fc) {
	//
	// ObjectName objectName = new ObjectName();
	// objectName = toMmsObjectName(new ObjectReference(reference), fc);
	//
	// return objectName;
	// }
	//
	public static ObjectName toMmsObjectName(
			ObjectReference reference, FunctionalConstraint fc) {
		// ObjectName objectName = new ObjectName();
		// Domain_specificSequenceType value = new
		// Domain_specificSequenceType();
		// value.setDomainId(new Identifier(reference.get(0)));
		// value.setItemId(new Identifier(getMMSItemId(reference, fc)));

		// objectName.selectDomain_specific(value);
		// return objectName;
		ObjectName.SubSeq_domain_specific value = null;
		
		try{
			value = new ObjectName.SubSeq_domain_specific(
				new BerVisibleString(reference.get(0)), new BerVisibleString(
						getMMSItemId(reference, fc)));
		}
		catch(UnsupportedEncodingException e){
			throw new IllegalAccessError();
		}
		
		return new ObjectName(null, value, null);
	}

	//
	// // for data sets, the FC is not added in the mms request (IEDScout and
	// TMW
	// // do not appear to use it)
	// public static ObjectName toMmsObjectNameDS(String dsRef) {
	//
	// int at = dsRef.indexOf('@');
	// ObjectName obj = new ObjectName();
	//
	// if (at < 0) {// persistent or configured
	// int slash = dsRef.indexOf('/');
	// String domainID = dsRef.substring(0, slash);
	// String itemID = dsRef.substring(slash + 1, dsRef.length()).replace(
	// '.', '$');
	// Domain_specificSequenceType value = new Domain_specificSequenceType();
	// value.setDomainId(new Identifier(domainID));
	// value.setItemId(new Identifier(itemID));
	// obj.selectDomain_specific(value);
	// }
	//
	// else {
	// obj.selectAa_specific(new Identifier(dsRef));
	// }
	//
	// return obj;
	//
	// }

	/**
	 * @param mmsItemId
	 * @return Example:<br>
	 *         Input: itemId = T1MMXU0$MX$TotW$mag$f or T1MMXU0$MX Result =
	 *         FunctionalConstraint.MX
	 */
	public static FunctionalConstraint getFunctionalConstraint(String mmsItemId) {

		int firstDollar = mmsItemId.indexOf('$');
		if (firstDollar >= 0) {
			int secondDollar = mmsItemId.indexOf('$', firstDollar + 1);
			if (secondDollar >= 0) {
				String fc = mmsItemId.substring(firstDollar + 1, secondDollar);
				return FunctionalConstraint.fromString(fc);
			}
			String fc = mmsItemId.substring(firstDollar + 1);
			return FunctionalConstraint.fromString(fc);
		}
		return null;
	}

	/**
	 * @param fc
	 * @return Example:<br>
	 *         value = E1Rly1/T1MMXU0.TotW.mag.f, fc = MX<br>
	 *         Result = T1MMXU0$MX$TotW$mag$f
	 */
	public static String getMMSItemId(ObjectReference reference,
			FunctionalConstraint fc) {

		// String referenceName = reference.toString();
		//
		// int firstSlash = referenceName.indexOf('/');
		//
		// if (firstSlash < 0) {
		// return "";
		// }
		//
		// int dotAfterSlash = referenceName.indexOf('.', firstSlash + 1);
		//
		// //get logical node name
		// String lnName = reference.get(2);
		//
		// if (dotAfterSlash >= 0) {
		// String fcStr = "XX"; // TODO check if fc = XX is valid for MMS SCSM
		// if (fc != null) {
		// fcStr = fc.toString();
		// }
		// return lnName + "$" + fcStr + "$" +
		// referenceName.substring(dotAfterSlash + 1).replace('.', '$');
		// }
		// if (fc != null) {
		// return lnName + "$" + fc.toString();
		// }
		// return lnName;

		if (reference.isLogicalDeviceRef()) {
			return "";
		}

		if (reference.isLogicalNodeRef()) {
			if (fc != null) {
				return reference.get(1) + "$" + fc.toString();
			}
			return reference.get(1);
		}

		if (reference.isDataRef()) {
			if (fc != null) {
				String id = reference.get(1) + "$" + fc.toString();
				for (int i = 0; i < reference.size() - 2; i++) {
					id = id + "$" + reference.get(i + 2);
				}
				return id;
			}// case where fc = null?
		}

		return "";
	}

}
