/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.scsm.mms.client;

import org.openiec61850.client.ClientACSI;
import org.openiec61850.client.ClientScsmSAP;
import org.openiec61850.common.ServiceError;
import org.openiec61850.jmms.iso.acse.ClientAcseSAP;

/**
 * @author Stefan Feuerhahn
 * @author Tobias Weidelt
 */
public class MmsScsmClientSAP implements ClientScsmSAP {

	static final int MINIMUM_PDU_SIZE = 1000;

	private int proposedMaxPduSize = 65000;
	public int proposedMaxServOutstandingCalling = 5;
	public int proposedMaxServOutstandingCalled = 5;
	public int proposedDataStructureNestingLevel = 10;
	public byte[] servicesSupportedCalling = new byte[] { 0x40 & 0xff, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
	public int servicesSupportedCallingNumBits = 85;

	public ClientAcseSAP acseSAP;

	public MmsScsmClientSAP() {
		acseSAP = new ClientAcseSAP();
	}

	/**
	 * Set the maximum MMS Pdu size in bytes. The default size is 65000.
	 * 
	 * @param maxPduSize
	 *            should be at least MINIMUM_PDU_SIZE
	 */
	public void setMaxPduSize(int maxPduSize) {
		if (maxPduSize >= MINIMUM_PDU_SIZE) {
			this.proposedMaxPduSize = maxPduSize;
		}
		else {
			throw new IllegalArgumentException("maximum size is too small");
		}
	}

	/*
	 * For the MMS-Mapping the ServerAccessPointReference should be of the form
	 * "ip:port" or "domain:port" or simply "ip" or "domain". The default port
	 * is 102. The AuthenticationParamter has no effect for the MMS-Mapping
	 * because Version 1.0 of IEC61850-8-1 does not support authentication.
	 */
	public ClientACSI associate(String serverAccessPointReference, String authenticationParameter, int connectTimeout)
			throws ServiceError, IllegalArgumentException {

		acseSAP.tSAP.setConnectTimeout(connectTimeout);

		return new MmsScsmClientAssociation(serverAccessPointReference, authenticationParameter, acseSAP,
				proposedMaxPduSize, proposedMaxServOutstandingCalling, proposedMaxServOutstandingCalled,
				proposedDataStructureNestingLevel, servicesSupportedCalling, servicesSupportedCallingNumBits);
	}

}
