//package org.openiec61850.scsm.mms.server;
//
//import java.util.List;
//
//import org.openiec61850.common.ACSIClass;
//import org.openiec61850.common.ServiceError;
//import org.openiec61850.common.model.Array;
//import org.openiec61850.common.model.ConstructedDataAttribute;
//import org.openiec61850.common.model.DataContainer;
//import org.openiec61850.common.model.DataObject;
//import org.openiec61850.common.model.FunctionalConstraint;
//import org.openiec61850.common.model.IModelNode;
//import org.openiec61850.common.model.LogicalDevice;
//import org.openiec61850.common.model.LogicalNode;
//import org.openiec61850.common.model.ObjectType;
//import org.openiec61850.server.Association;
//import org.openiec61850.server.ServerACSI;
//import org.openiec61850.server.scsm.SCSMConnectionHandler;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;
//
///**
// * The idea behind this class is: Using a serice implementation according to
// * IEC61850-7.2 with the current {@link MMSConnectionHandler}. Due to the way
// * the MMS stack works, the service calls are breaking the 7.2 standard. This
// * class aims to resolve these inconsistencies.
// * 
// * For example, if a service GetDataDirectory(only allowed to target DataObjects
// * in 7.2) is received, if it targets a Logical Node the service request is
// * broken up and will be handled with several request internally.
// * 
// * @author ull
// * 
// */
//public class MMSGenericACSIAdapter implements ServerACSI {
//	private final static Logger LOG = LoggerFactory.getLogger(MMSGenericACSIAdapter.class);
//
//	private ServerACSI genericACSI;
//
//	public MMSGenericACSIAdapter(ServerACSI genericACSI) {
//		if (genericACSI != null) {
//			this.genericACSI = genericACSI;
//		}
//		else {
//			LOG.error("Can not initialize ACSIAdapter without ACSI!");
//		}
//	}
//
//	public Association associate(SCSMConnectionHandler clientHandler, String auth) throws ServiceError {
//		LOG.trace("Associate:" + auth);
//		Association associate = genericACSI.associate(clientHandler, auth);
//		return associate;
//	}
//
//	public void abort(Association association, String reason) throws ServiceError {
//		LOG.trace("Abort:" + reason);
//		genericACSI.abort(association, reason);
//	}
//
//	public void release(Association association) throws ServiceError {
//		LOG.trace("Release");
//		genericACSI.release(association);
//	}
//
//	/**
//	 * No unexpected parameters possible.
//	 * 
//	 * @return A list of logical devices , e.g. ["LD1", "LD2"], or an empty
//	 *         array
//	 */
//	public List<String> getServerDirectory(Association association, ServerDirectoryType dirType) throws ServiceError {
//		LOG.trace("GetServerDirectory");
//		List<String> serverDirectory = genericACSI.getServerDirectory(association, dirType);
//		return serverDirectory;
//	}
//
//	/**
//	 * 
//	 * @param ldName
//	 *            The targeted logical device
//	 * @return a list of logical nodes contained within the logical device, e.g.
//	 *         ["LN1", "LN2"], or an empty array
//	 */
//	public List<String> getLogicalDeviceDirectory(Association association, String ldName) throws ServiceError {
//		LOG.trace("GetLogicalDEviceDirectory:" + ldName);
//		List<String> logicalDeviceDirectory = genericACSI.getLogicalDeviceDirectory(association, ldName);
//		return logicalDeviceDirectory;
//	}
//
//	/**
//	 * 
//	 * 
//	 * @param lnRef
//	 *            The reference of the targeted logical node
//	 * @param acsiClass
//	 *            The type of content of the target logical node to return, e.g.
//	 *            Data Set, Data
//	 * @return
//	 */
//	public List<String> getLogicalNodeDirectory(Association association, String lnRef, ACSIClass acsiClass)
//			throws ServiceError {
//		LOG.trace("GetLogicalNodeDirectory: " + lnRef + " - " + acsiClass);
//		List<String> logicalNodeDirectory = genericACSI.getLogicalNodeDirectory(association, lnRef, acsiClass);
//		return logicalNodeDirectory;
//	}
//
//	/**
//	 * MMS clients sometimes address model nodes other than data objects. <br>
//	 * <u>Source Data:</u> <br>
//	 * name of SubDo and name of SubDA of a specified DataObject <br>
//	 * <br>
//	 * See 61850v2 7.2 11.4.4 <br>
//	 * See 61850v2 8.1 13.2.3
//	 * 
//	 * @param dataRef
//	 *            According to 7.2, dataRef may only address DataObjects
//	 * @return The complete sub-tree with root at the addressed data object.
//	 */
//	public DataContainer getDataDirectory(Association association, String dataRef) throws ServiceError {
//		LOG.trace("GetDataDirectory: " + dataRef);
//
//		if (!isRefDO(dataRef)) {
//			LOG.warn("GetDataDirectory with illegal reference " + dataRef);
//			return null;
//		}
//
//		DataContainer dataDirectory = genericACSI.getDataDirectory(association, dataRef);
//		if (!(dataDirectory instanceof DataObject)) {
//			LOG.warn("GetDataDirectory must return a DataObject");
//			throw new ServiceError(ServiceError.ACCESS_VIOLATION);
//		}
//
//		DataObject object = (DataObject) dataDirectory;
//
//		// TODO FIXME
//		for (DataContainer dataObject_ : object.getSubDataObjects()) {
//			DataObject dataObject = (DataObject) dataObject_;
//			// Recursive...
//		}
//		for (DataContainer dataAttribute_ : object.getDataAttributes()) {
//			ConstructedDataAttribute dataAttribute = (ConstructedDataAttribute) dataAttribute_;
//			IModelNode dataDefinition = genericACSI.getDataDefinition(association, dataAttribute.getReference()
//					.toString());
//
//		}
//
//		return dataDirectory;
//	}
//
//	/**
//	 * According to 61850v2 - 8.1 - 13.2.4 : maps to GetDataDirectory
//	 * 
//	 * @param ref
//	 *            addresses a Data Object
//	 * @return The subtree with root of the addressed reference.
//	 */
//	public IModelNode getDataDefinition(Association association, String ref) throws ServiceError {
//		LOG.trace("GetDataDefinition: " + ref);
//
//		IModelNode result = null;
//
//		if (isRefLD(ref)) {
//			result = getDataDefinitionLD(association, ref);
//		}
//		else if (isRefLN(ref)) {
//			result = getDataDefinitionLN(association, ref);
//		}
//		else if (isRefDO(ref)) {
//			// call directly
//			result = genericACSI.getDataDefinition(association, ref);
//			result = getDataDirectory(association, ref);
//		}
//		else if (isRefDODA(ref)) {
//
//			boolean refSubDO = isRefSubDO(association, ref);
//			LOG.warn("IsRefSubDO: " + ref + ": " + refSubDO);
//			// FIXME TODO XXX WHAT TO DO THIS MIGHT BE A DO AS WELL! HELP!
//			result = getDataDefinitionDA(association, ref);
//		}
//
//		if (result == null) {
//			LOG.warn("getDataDefinition EVIL null!");
//		}
//
//		return result;
//	}
//
//	/**
//	 * 
//	 * @param association
//	 * @param ref
//	 * @return
//	 */
//	private ConstructedDataAttribute getDataDefinitionDA(Association association, String ref) throws ServiceError {
//		if (!isRefDODA(ref)) {
//			LOG.warn("GetDataDefinitionDA with maybe with illegal ref, only DA allowed! " + ref);
//		}
//
//		// 1.)Find parent of desired DataAttribute
//		String parentRef = ref.substring(0, ref.lastIndexOf("."));
//		String daName = ref.substring(ref.lastIndexOf(".") + 1, ref.length());
//		LOG.debug("GetDataDefinitionDA with " + parentRef + " - " + daName);
//
//		// 2.)Get parent and "extract" desired Data Attribute
//		DataObject parent = (DataObject) getDataDefinition(association, parentRef);
//
//		DataContainer dataContainer = null;
//		if (parent != null) {
//			dataContainer = parent.getDataAttribute(daName);
//		}
//
//		if (dataContainer instanceof ConstructedDataAttribute) {
//			ConstructedDataAttribute da = (ConstructedDataAttribute) dataContainer;
//			return da;
//		}
//		else {
//			LOG.warn("Could not find DataAttribute with ref " + ref + ", instead found " + dataContainer);
//			return null;
//		}
//	}
//
//	/**
//	 * DataDefinition which (illegaly) addresses a LogicalDevice
//	 * 
//	 * @param ref
//	 * @return
//	 */
//	private LogicalDevice getDataDefinitionLD(Association association, String ref) throws ServiceError {
//		if (!isRefLD(ref)) {
//			LOG.warn("GetDataDefinitionLD with illegal ref " + ref);
//			return null;
//		}
//
//		LogicalDevice ld = new LogicalDevice(ref);
//		// 1.) Get content of LD
//		List<String> lnNameList = getLogicalDeviceDirectory(association, ref);
//
//		// 2.) For each LN: Get subtree and add to LD to return
//		for (String lnName : lnNameList) {
//			LogicalNode ln = getDataDefinitionLN(association, lnName);
//			ld.addLogicalNode(ln);
//		}
//
//		return ld;
//	}
//
//	/**
//	 * DataDefinition which (illegaly) addresses a LogicalNode
//	 * 
//	 * @param association
//	 * @param ref
//	 * @return
//	 */
//	private LogicalNode getDataDefinitionLN(Association association, String ref) throws ServiceError {
//		if (!isRefLN(ref)) {
//			LOG.warn("GetDataDefinitionLD with illegal ref " + ref);
//			return null;
//		}
//
//		// 1.) Get Logical Node directory
//		LogicalNode lNode = new LogicalNode(ref);
//		List<String> doNameList = getLogicalNodeDirectory(association, ref, ACSIClass.DATA_OBJECT);
//
//		// 2.) For each 'DataObject': Get subtree and merge
//		for (String doName : doNameList) {
//			// With a dataObject reference, we can legally call getDataDirectory
//			// directly
//			IModelNode data = getDataDefinition(association, lNode.getReference() + "." + doName);
//			if (data instanceof DataObject) {
//				lNode.addDataObject((DataObject) data);
//			}
//			else {
//				if (data != null)
//					LOG.warn("Data ist of type " + data.getClass());
//				else
//					LOG.warn("Data is null");
//			}
//		}
//
//		// 3.) return final result
//		return lNode;
//	}
//
//	/**
//	 * MMS - expects subTree starting at "ref"<br>
//	 * GeneralACSI - supplies DataAttributeValues with ref as parent* somewhere
//	 * up the tree
//	 * 
//	 * @param ref
//	 * @param fc
//	 * @return
//	 */
//	public DataContainer getDataValues(Association association, String ref, FunctionalConstraint fc)
//			throws ServiceError {
//		LOG.trace("GetDataValues: " + ref + " - " + fc.toString());
//
//		if (isRefDO(ref) || isRefDODA(ref)) {
//			DataContainer dataValues = genericACSI.getDataValues(association, ref, fc);
//			return dataValues;
//		}
//		else if (isRefLN(ref)) {
//			Array dataValuesLN = getDataValuesLN(association, ref, fc);
//			return dataValuesLN;
//		}
//		else {
//			LOG.warn("wrong ref : " + ref);
//		}
//
//		FunctionalConstraint unused = (fc == FunctionalConstraint.ST) ? FunctionalConstraint.CO
//				: FunctionalConstraint.ST;
//		ConstructedDataAttribute noResult = new ConstructedDataAttribute("FooBar", unused);
//
//		return noResult;
//	}
//
//	private Array getDataValuesLN(Association association, String ref, FunctionalConstraint fc) throws ServiceError {
//		Array resultArr = new Array(ref, fc, ObjectType.DATA_OBJECT);
//
//		List<String> dataNames = getLogicalNodeDirectory(association, ref, ACSIClass.DATA_OBJECT);
//
//		for (String dataName : dataNames) {
//			DataContainer dataValues = getDataValues(association, ref + "." + dataName, fc);
//			if (dataValues != null)
//				resultArr.append(dataValues);
//			else
//				LOG.warn("returned null for getDataValues " + ref + "-" + fc.toString());
//		}
//
//		return resultArr;
//	}
//
//	public DataContainer getDataSetDirectory(Association association, String dataSetname) throws ServiceError {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	public void getDataSetValues(Association association, List<DataContainer> dataSetMembers) throws ServiceError {
//		// TODO Auto-generated method stub
//
//	}
//
//	public void setDataValues(Association association, DataContainer container) throws ServiceError {
//		// TODO Auto-generated method stub
//	}
//
//	public void selectEditSG(Association association, String ref, int sGN) throws ServiceError {
//		// TODO Auto-generated method stub
//	}
//
//	public void createDataSet(Association association, String dataSetName, List<String> dsMemberList,
//			List<FunctionalConstraint> fc) throws ServiceError {
//		// TODO Auto-generated method stub
//	}
//
//	public int[] deleteDataSet(Association association, String dataSetName) throws ServiceError {
//		// TODO Auto-generated method stub
//		return null;
//	}
//
//	/**
//	 * Assumes a standard 61850 reference: LD/xxx_LN.DO.SubDO.DA.SubDA
//	 */
//	private boolean isRefLD(String ref) {
//		return !ref.contains("/");
//	}
//
//	/**
//	 * Assumes a standard 61850 reference: LD/xxx_LN.DO.DA.SubDA
//	 */
//	private boolean isRefLN(String ref) {
//		boolean result = ref.contains("/") && !ref.contains(".");
//		return result;
//	}
//
//	/**
//	 * Assumes a standard 61850 reference: LD/xxx_LN.DO.SubDO.DA.SubDA
//	 */
//	private boolean isRefDO(String ref) {
//		return ref.split("\\.").length == 2;
//	}
//
//	/**
//	 * Assumes a standard 61850 reference: LD/xxx_LN.DO.SubDO.DA.SubDA
//	 */
//	private boolean isRefDODA(String ref) {
//		// TODO FIXME XXX Can not be trusted
//		return ref.split("\\.").length >= 3;
//	}
//
//	/**
//	 * 
//	 * @param assoc
//	 * @param ref
//	 * @return
//	 */
//	private boolean isRefSubDO(Association assoc, String ref) {
//		if (ref == null)
//			return false;
//		if (!isRefDODA(ref))
//			return false;
//
//		// Split the complete ref to be able to access the parts
//		// [LD/LN].[DO].[SubDO].[SubDO].[SubDa].[rest]
//		// # 0 1 2 3 4 5
//		String[] layers = ref.split("\\.");
//
//		if (!isRefLN(layers[0]))
//			return false;
//
//		for (int i = 1; i < layers.length; i++) {
//
//			// Now rebuild the current level
//			String currentRef = layers[0];
//			for (int j = 1; j <= i; j++) {
//				currentRef += "." + layers[j];
//			}
//
//			LOG.warn("Checking subRef " + currentRef);
//
//			// Check if the current level is still a DO
//			try {
//				DataContainer dataDirectory = genericACSI.getDataDirectory(assoc, currentRef);
//				// IModelNode child = dataDirectory.getChild(layers[i+1]);
//				if (!(dataDirectory instanceof DataObject))
//					return false;
//
//			} catch (ServiceError e) {
//				return false;
//			}
//		}
//
//		return true;
//	}
//
// }


